@@ -19,7 +19,7 @@ static int perly_unpacking = 1; /* state variable */
  * Get the width of a string column in an ASCII or binary table
  */
 long column_width(fitsfile * fptr, int colnum) {
-  int hdutype, status=0, tfields;
+  int hdutype, status=0, tfields, dispwidth;
   long repeat, size;
   long start_col,end_col; /* starting and ending positions for ASCII tables */
   long rowlen, nrows, *tbcol;
@@ -58,23 +58,12 @@ long column_width(fitsfile * fptr, int colnum) {
     size = end_col - start_col;
     break;
 
-    /* Get the typechar parameter, which should be of form 'An', where
-     * n is an the width of the field
-     */
   case BINARY_TBL:
-    fits_get_bcolparms(
-		       fptr,colnum,NULL,NULL,typechar,&repeat,NULL,NULL,
-		       NULL,NULL,&status
-		       );
+    fits_get_col_display_width(fptr, colnum, &dispwidth, &status);
     check_status(status);
-    if (typechar[0] != 'A') { /* perhaps variable size? */
-      fits_read_key_lng(fptr,"NAXIS2",&rowlen,NULL,&status);
-      check_status(status);
-      size  = rowlen+1;
-    }
-    else
-      size = repeat;
+    size = dispwidth;
     break;
+
   default:
     croak("column_width() - unrecognized HDU type (%d)",hdutype);
   }
@@ -461,6 +450,9 @@ void pack_element(SV* work, SV** arg, int datatype) {
   unsigned long ulscalar;
   long lscalar;
   LONGLONG llscalar;
+#ifdef TULONGLONG
+  ULONGLONG ullscalar;
+#endif
   float fscalar;
   double dscalar;
 
@@ -519,6 +511,12 @@ void pack_element(SV* work, SV** arg, int datatype) {
       llscalar = arg ? SvIV(*arg) : 0;
       sv_catpvn(work, (char *) &llscalar, size);
       break;
+#ifdef TULONGLONG
+    case TULONGLONG:
+      ullscalar = arg ? SvIV(*arg) : 0;
+      sv_catpvn(work, (char *) &ullscalar, size);
+      break;
+#endif
     case TCOMPLEX:
       size /= 2;
     case TFLOAT:
@@ -662,8 +660,16 @@ void unpackNDll ( SV * arg, void * var, int ndims, LONGLONG *dims,
 void unpackND (SV* arg, void* var, int ndims, long *dims,
 	       int datatype, int perlyunpack)
 {
-  LONGLONG* dimsll = malloc(ndims*sizeof(LONGLONG));
+  LONGLONG* dimsll;
   int i;
+
+  if (1==ndims) {
+    unpack1D(arg, var, dims[0], datatype, perlyunpack);
+    return;
+  }
+
+  dimsll = malloc(ndims*sizeof(LONGLONG));
+
   for (i=0; i<ndims; ++i)
     dimsll[i] = dims[i];
   unpackNDll(arg, var, ndims, dimsll, datatype, perlyunpack);
@@ -725,6 +731,10 @@ void unpackScalar(SV * arg, void * var, int datatype) {
     sv_setiv(arg,(IV)(*(long *)var)); break;
   case TLONGLONG:
     sv_setiv(arg,(IV)(*(LONGLONG *)var)); break;
+#ifdef TULONGLONG
+  case TULONGLONG:
+    sv_setiv(arg,(IV)(*(ULONGLONG *)var)); break;
+#endif
   case TFLOAT:
     sv_setnv(arg,(double)(*(float *)var)); break;
   case TDOUBLE:
@@ -762,6 +772,9 @@ void unpack1D (SV* arg, void* var, LONGLONG n, int datatype, int perlyunpack)
   unsigned long * ulvar;
   long * lvar;
   LONGLONG * llvar;
+#ifdef TULONGLONG
+  ULONGLONG * ullvar;
+#endif
   float * fvar;
   double * dvar;
   SV *tmp_sv[2];
@@ -838,6 +851,13 @@ void unpack1D (SV* arg, void* var, LONGLONG n, int datatype, int perlyunpack)
     for(i=0; i<m; i++)
       av_store(array, i, newSViv( (IV)llvar[i] ));
     break;
+#ifdef TULONGLONG
+  case TULONGLONG:
+    ullvar = (ULONGLONG *) var;
+    for(i=0; i<m; i++)
+      av_store(array, i, newSViv( (IV)ullvar[i] ));
+    break;
+#endif
   case TFLOAT:
     fvar = (float *) var;
     for(i=0; i<m; i++)
@@ -961,6 +981,10 @@ int sizeof_datatype(int datatype) {
     return sizeof(long);
   case TLONGLONG:
     return sizeof(LONGLONG);
+#ifdef TULONGLONG
+  case TULONGLONG:
+    return sizeof(ULONGLONG);
+#endif
   case TFLOAT:
     return sizeof(float);
   case TDOUBLE:
